__all__ = [
    'GenericInitiatePaymentView',
    'InitiatePaymentView'
]
from django.shortcuts import render, Http404
from django.conf import settings as django_settings
from django.views import View

from paytm import conf as paytm_conf


class GenericInitiatePaymentView(View):
    conf = paytm_conf

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.request = None

    def get_merchant_credentials(self):
        """
        :return: dict({
            MID
            WEBSITE
            INDUSTRY_TYPE_ID
        })
        """

        return self.conf.merchant_credentials

    def get_channel(self):
        """
        :return: str('WEB' | 'WAP')
        """
        return 'WEB'

    def get_payload(self):
        pass

    def post(self, request):
        self.request = request

        return render(request, 'paytm/checkout/redirect.html', self.get_payload())


class InitiatePaymentView(GenericInitiatePaymentView):
    """Wrapper for testing"""

    def get(self, request):
        if not django_settings.DEBUG:
            raise Http404

        return render(request, 'paytm/checkout/index.html')



from django.urls import path
from .views import *

urlpatterns = [
    path('', InitiatePaymentView.as_view(), name='index'),
    path('initiate/', InitiatePaymentView.as_view(), name='initiate'),

]

app_name = 'paytm:checkout'



"""Only to getting configuration set in project's settings.py"""
from django.conf import settings as django_settings

from paytm.exceptions import IncorrectConfiguration


class Configuration(object):
    """Class to get configuration and then testing its format correctness"""

    MID = ''
    KEY = ''
    INDUSTRY = 'Retail'
    WEBSITE = 'WEBSTAGING'
    DEBUG = True

    CHANNEL_WEBSITE = 'WEB'
    CHANNEL_MOBILE_APP = 'WAP'
    STAGING_DOMAIN = 'securegw-stage.paytm.in'
    PRODUCTION_DOMAIN = 'securegw.paytm.in'

    settings_mapping = {
        # Key: (attr, type, required, defaults)
        'PAYTM_MERCHANT_ID': ('MID', str, True, MID),
        'PAYTM_MERCHANT_KEY': ('KEY', str, True, KEY),
        'PAYTM_INDUSTRY': ('INDUSTRY', str, True, INDUSTRY),
        'PAYTM_WEBSITE': ('WEBSITE', str, True, WEBSITE),

        'PAYTM_DEBUG': ('DEBUG', bool, False, DEBUG),

        'PAYTM_CHANNEL_WEBSITE': ('CHANNEL_WEBSITE', str, False, CHANNEL_WEBSITE),
        'PAYTM_CHANNEL_MOBILE_APP': ('CHANNEL_MOBILE_APP', str, False, CHANNEL_MOBILE_APP),

        'PAYTM_STAGING_DOMAIN': ('STAGING_DOMAIN', str, False, STAGING_DOMAIN),
        'PAYTM_PRODUCTION_DOMAIN': ('PRODUCTION_DOMAIN', str, False, PRODUCTION_DOMAIN)
    }

    def __init__(self):
        # Get the values from settings and then set attribute
        for key in self.settings_mapping:
            setattr(
                self,
                self.settings_mapping[key][0],
                self._get(key)
            )

        if self.DEBUG:
            self.DOMAIN = self.STAGING_DOMAIN
        else:
            self.DOMAIN = self.PRODUCTION_DOMAIN

        # deleting for safety
        delattr(self, 'STAGING_DOMAIN')
        delattr(self, 'PRODUCTION_DOMAIN')

        self.merchant_credentials = {
            "MID": self.MID,
            "WEBSITE": self.WEBSITE,
            "INDUSTRY_TYPE_ID": self.INDUSTRY,
        }

    def _get(self, name):
        """Gets a particular settings from django_settings and checks its correctness"""

        attr_name, attr_type, required, default = self.settings_mapping[name]

        if required:
            try:
                value = getattr(django_settings, name)
            except AttributeError:
                # Raise
                raise IncorrectConfiguration('Please, add {} in your settings.py'.format(name))
        else:
            value = getattr(django_settings, name, default)

        if type(value) != attr_type:
            raise IncorrectConfiguration(
                '{} has incorrect type, required {} got {}'.format(name, attr_type, type(value))
            )

        return value



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / 'README.md').read_text()

# This call to setup() does all the work
setup(
    name='django-paytm-checkout',
    version='0.0.2',
    description='A simple and modular django app to help with Paytm checkout and custom checkout',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/Faisal-Manzer/django-paytm-checkout',
    author='Faisal Manzer',
    author_email='faisal_manzer@yahoo.in',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    packages=find_packages(include=('paytm*',)),
    include_package_data=True,
    install_requires=['django', 'djangorestframework'],
    download_url='https://github.com/Faisal-Manzer/django-paytm-checkout/archive/0.0.1.tar.gz',
    entry_points={
    },
)
